﻿using System;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using STFXmlParser;
using Selections;
using CTNet;
using PdfDocument;
using System.Text.RegularExpressions;
using System.Linq;

namespace CalcHelper
{
    public partial class Form1 : Form
    {
        /*
         * This activates the orphanBoxesBtn to inform the user that orphan
         * boxes have been detected either at CalcHelper launch or after a
         * CalcHelper refresh, or the button is deactivated.
         * 
         * Called From:
         * 
         * SetupPdf1()
         * refreshBtn_Click()
         */
        public void ProcessOrphanBoxesButton()
        {
            if (docFile.orphanBoxes.Count > 0)
            {
                if ((ctClient != null) && (ctClient.boxSelClient == boxSelDest.chkBoxCtrl))
                {
                    ctClient.orphanBtnState = btnState.on;
                }
                else if (orphanBoxesBtn.Enabled == false)
                {
                    orphanBoxesBtn.Enabled = true;
                    orphanBoxesBtn.Image = CalcHelper.Properties.Resources.orphanBoxesOn;
                    orphanBoxesBtn.ToolTipText = "Orphan Boxes Detected";
                }
            }
            else
            {
                if ((ctClient != null) && (ctClient.boxSelClient == boxSelDest.chkBoxCtrl))
                {
                    ctClient.orphanBtnState = btnState.off;
                }
                else if (orphanBoxesBtn.Enabled == true)
                {
                    orphanBoxesBtn.Enabled = false;
                    orphanBoxesBtn.Image = CalcHelper.Properties.Resources.orphanBoxesOff;
                    orphanBoxesBtn.ToolTipText = "No Orphan Boxes Detected";
                }
            }
        }

        /*
         * Called From:
         * 
         *      SetupPdf1()
         *      SetupPdf2()
         *      
         * We just opened a file and set the box page lists in Field.cs
         * which are lists of page numbers that pertain to boxes such as
         * checkboxes, code39 barcode boxes, etc.. We will allow the user
         * to be able to limit navigation to these. We also built a page
         * list for pages that have boxes on them.
         * 
         * All page navigation menu items are disabled in the designer as
         * some documents may not have any 2D barcodes on them for example.
         * Any that are enabled will allow the user to navigate to just
         * those pages so they do not have to 'page-flip' looking for say,
         * checkboxes.
         */
        public void EnablePageNavMenuItems(string pdfId, ref PageArray pa)
        {
            if (pa == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null PageArray argument:\n\nEnablePageNavMenuItems() received a null pa argument."))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            switch (pdfId)
            {
                case ("pdf1"):

                    /*
                     * Pages with boxes on them. Checking for > 0 due to the RARE case
                     * of only 1 page with one box which is box1_65535 which I do not
                     * store in the dictionary, so this list would be empty. I do not
                     * count box1_65535 as a box'.
                     */
                    if (pa.GetBoxedPageListCount("boxedpages") > 0)
                    {
                        boxedPageToolStripMenuItem.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("checkbox") > 0)
                    {
                        chkboxPageToolStripMenuItem.Enabled = true;

                        if (ctClient != null)
                        {
                            // Do not need to do this for "pdf2" case.
                            ctClient.docHasCheckboxes = true;
                        }
                    }

                    if (pa.GetBoxedPageListCount("code39") > 0)
                    {
                        code39ToolStripMenuItem.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("int2of5") > 0)
                    {
                        int2of5ToolStripMenuItem.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("pdf417") > 0)
                    {
                        pdf417ToolStripMenuItem.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("qrcode") > 0)
                    {
                        qrCodesToolStripMenuItem.Enabled = true;
                    }

                    break;

                case ("pdf2"):

                    if (pa.GetBoxedPageListCount("boxedpages") > 0)
                    {
                        boxedPageToolStripMenuItem2.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("checkbox") > 0)
                    {
                        chkboxToolStripMenuItem2.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("code39") > 0)
                    {
                        code39ToolStripMenuItem2.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("int2of5") > 0)
                    {
                        int2of5ToolStripMenuItem2.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("pdf417") > 0)
                    {
                        pdf417ToolStripMenuItem2.Enabled = true;
                    }

                    if (pa.GetBoxedPageListCount("qrcode") > 0)
                    {
                        qrCodesToolStripMenuItem2.Enabled = true;
                    }

                    break;
            }
        }

        /*
         * Called From:
         * 
         *      ctClient_CtClientEvent()
         *      
         * This handles calc tool events ctEvent.docLock and ctEvent.docUnLock.
         * docLock means we are 'locking' the Calc Helper in regards to sending
         * messages/data to the Calc Tool Server since the file is not the current
         * editor file, somebody either closed the file or opened a different file.
         * 
         * unLock as the name implies, reverses the locking, we are now in synch with
         * the editor file so we can send commands/data to the editor via the Calc Tool
         * Server.
         * 
         * In either case, the Calc Tool Server can send messages to Calc Helper such as
         * "hey, they just opened your file again...".
         * 
         * Arguments:
         * 
         *  mode - 'true' -> docLock, 'false' -> docUnLock.
         */
        public void ProcessDocLock(bool mode)
        {
            if (mode == true)
            {
                /*
                 * Idiot check to be safe, even though we got here as a
                 * result of a ctClient event.
                 */
                if (ctClient != null)
                {
                    if (ctClient.boxSelClient == boxSelDest.document)
                    {
                        ProcessBoxSelMode(boxSelDest.off);
                    }
                    else if (ctClient.boxSelClient == boxSelDest.chkBoxCtrl)
                    {
                        ProcessChkBoxCtrlMode(false, true, true);
                    }
                    else if (boxSel.rightCtrlBoxSel != null)
                    {
                        string argStr = string.Format("Cannot unselect {0}.", boxSel.rightCtrlBoxSel.box);

                        if (!Pa1OrPdf1AreNull("ProcessDocLock", argStr))
                        {
                            // Should reset this selection...being picky.
                            boxSel.ResetSelections(boxSelTypes.rightCtrlSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                            RefreshPanels();
                        }
                    }

                    ctClient.refreshBtnState = (refreshBtn.Enabled == true ? btnState.on : btnState.off);
                    ctClient.orphanBtnState = (orphanBoxesBtn.Enabled == true ? btnState.on : btnState.off);
                }

                // Lock'em Danil! [5 oh...]
                if (toolBelt != null)
                {
                    toolBelt.Close();
                }

                vsCodeIconLbl.Image = CalcHelper.Properties.Resources.VscodeYellow;
                vsCodeIconLbl.ToolTipText = "Not Connected To Vscode File";

                refreshBtn.Enabled = false;
                orphanBoxesBtn.Enabled = false;

                toolsToolStripMenuItem.Enabled = false;
                docBoxSelToolStripMenuItem.Enabled = false;
            }
            else
            {
                vsCodeIconLbl.Image = CalcHelper.Properties.Resources.VscodeGreen;
                vsCodeIconLbl.ToolTipText = "Connected To Vscode File";

                toolsToolStripMenuItem.Enabled = true;
                docBoxSelToolStripMenuItem.Enabled = true;

                /*
                 * Idiot check to be safe, even though we got here as a
                 * result of a ctClient event.
                 */
                if (ctClient != null)
                {
                    refreshBtn.Enabled = (ctClient.refreshBtnState == btnState.on ? true : false);
                    orphanBoxesBtn.Enabled = (ctClient.orphanBtnState == btnState.on ? true : false);
                }
            }
        }

        /*
         * This proceses the boxSelModeDropDwnBtn's menu selections: Document,
         * Checkbox Control, Dataflow Retriever, and Box Number Magnification.
         * See the 'Called from' section below for other reasons why we are
         * calling this function.
         * 
         * The displayed image and tooltip text are set to reflect the Box
         * Selection mode. We also set the ctClient's box selection
         * destination var so it can send box selections off to the proper
         * recipient, if any.
         * 
         * Called from:
         * 
         *  docBoxSelToolStripMenuItem_Click()
         *  boxNumMagToolStripMenuItem_Click()
         *  ProcessDocLock()
         *  ProcessChkBoxCtrlMode()
         *  ProcessDFRetMode()
         */
        public void ProcessBoxSelMode(boxSelDest boxSelDest)
        {
            // Caller has ensured that ctClient is not null, which it should never be...

            /*
             * Disable the Box Number Magnification tooltip if currently enabled - 
             * the mouseLeave() event handler for the picture boxes should have done
             * this...but....
             */
            if (ctClient.boxSelClient == boxSelDest.boxNumMag)
            {
                this.boxNumMagToolTip.Active = false;
            }

            ctClient.boxSelClient = boxSelDest;

            switch (boxSelDest)
            {
                case (boxSelDest.chkBoxCtrl):

                    /*
                     * Checkbox Control tool has successfully requested to receive check
                     * box selections.
                     */
                    boxSelModeDropDwnBtn.Image = CalcHelper.Properties.Resources.boxSelChkBoxCtrl;
                    boxSelModeDropDwnBtn.ToolTipText = "Box Selection Mode: Checkbox Control Tool";
                    break;

                case (boxSelDest.dfRetriever):

                    /*
                     * Dataflow Retriever tool has successfully requested to receive box selections.
                     */
                    boxSelModeDropDwnBtn.Image = CalcHelper.Properties.Resources.boxSelDFRet;
                    boxSelModeDropDwnBtn.ToolTipText = "Box Selection Mode: Dataflow Retriever Tool";
                    break;

                case (boxSelDest.document):

                    /*
                     * User wants to start sending box selections to the Vscode file.
                     */
                    boxSelModeDropDwnBtn.Image = CalcHelper.Properties.Resources.boxSelDoc;
                    boxSelModeDropDwnBtn.ToolTipText = "Box Selection Mode: Document's Vscode File";
                    break;

                case (boxSelDest.boxNumMag):

                    /*
                     * User wants to start selecting boxes to see the numbers magnified:
                     * 
                     * The below empty string wipes the last magnified box number from
                     * the stool tip - thanks to 2-ply technology!
                     */
                    this.boxNumMagToolTip.Active = true;

                    if (pdf1 != null)
                    {
                        pdf1.mLassoDown = false;
                        pdf1.mLassoMove = false;
                        boxNumMagToolTip.SetToolTip(this.pictureBox1, "");
                    }

                    if (pdf2 != null)
                    {
                        pdf2.mLassoDown = false;
                        pdf2.mLassoMove = false;
                        boxNumMagToolTip.SetToolTip(this.pictureBox2, "");
                    }

                    boxSelModeDropDwnBtn.Image = CalcHelper.Properties.Resources.boxSelBoxNumMag;
                    boxSelModeDropDwnBtn.ToolTipText = "Box Selection Mode: Box Number Magnification";
                    break;

                default:

                    /*
                     * Box Selection mode is being turned 'Off'.
                     */
                    boxSelModeDropDwnBtn.Image = CalcHelper.Properties.Resources.boxSelOff;
                    boxSelModeDropDwnBtn.ToolTipText = "Box Selection Mode: No Selection";
                    break;
            }

            if (!Pa1OrPdf1AreNull("ProcessBoxSelMode", "Selection resets will not be possible."))
            {
                /*
                 * Reset all selections, basically erase the chalkboard.
                 */
                boxSel.ResetSelections(boxSelTypes.allSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                RefreshPanels();
            }
        }

        /*
         * This handles either the switching to 'Checkbox Control' box selection
         * mode or switching from 'Checkbox Control' box selection mode.
         * 
         * We switch to this mode only as a result of a request from the Checkbox
         * Control tool which can only be made if the document is in an 'unlocked'
         * state [ctClient.vscodeLock].
         * 
         * An 'unlocked' state means the file that Calc Helper launched from is still
         * the current editor file. If the current editor file changes then Calc Helper
         * is placed in a 'locked' state regarding sending info to that file, like
         * box selections, commands to move to a location in the editor file, etc..
         * 
         * Checkbox Control is also launched from an editor file - this is the only
         * way Checkbox Control can communicate with Calc Helper, it can not do this
         * if launched outside of an editor. For Checkbox Control to be able to
         * communicate with Calc Helper they must have been launched from the same
         * file.
         * 
         * Called From:
         * 
         *  ProcessDocLock()
         *  docBoxSelToolStripMenuItem_Click()
         *  chkBoxCtrlToolStripMenuItem_Click()
         *  ctClient_CtClientEvent()
         *  boxNumMagToolStripMenuItem_Click()
         * 
         * Arguments:
         * 
         *  status - 'true' means switching to, 'false' means switching from.
         *  
         *  informTool - 'true' means tell ctClient to send message to Checkbox Control
         *      that it will no longer be getting Calc Helper checkbox selections. Used
         *      only when the 'turning off' of checkbox selections does not come from
         *      the Checkbox Control tool itself, and only used when the 'status'
         *      argument is 'false'.
         *  
         *  setBoxSelMode - 'true' means we call ProcessBoxSelMode(), 'false' means we do
         *      not call it because we will be calling it for a different box selection mode
         *      so no sense in setting up for one mode then immediately change to another mode.
         *      
         * Switching To 'Checkbox Control' box selection mode:
         * 
         *      We enable the 'Checkbox Control' menu item in the 'Box Selection Mode'
         *          drop down menu button list. Users will now be able to turn this
         *          mode off from Calc Helper.
         *          
         *      We set page navigation mode to pages with checkboxes to make it easier
         *          for the user to select them [if not already in that mode].
         *          We do this for both display panels when a split screen.
         *          
         *      We reflect the switch to 'Checkbox Control' box selection mode in the
         *          'Box Selection Mode' drop down button's tooltip text and image if
         *          the 'setBoxSelMode' arg is 'true'.
         *          
         *      We set the ctClient's box selection destination to 'Checkbox Control'
         *      if the 'setBoxSelMode' arg is 'true'.
         *          
         * Switching From 'Checkbox Control' box selection mode (i.e. no longer doing it):
         * 
         *      These are the scenarios of how we got to turning this setting off.
         *  
         *          1) We receive from ctClient an accepted request from another tool
         *              for box selections. Accepted means both the other tool
         *              and Calc Helper need to be linked to the same document file.
         *              ctClient accepts or rejects.
         *      
         *          2) We receive from ctClient notification that the Checkbox Control
         *              tool no longer wants checkbox selections from Calc Helper.
         *              
         *          3) Calc Helper user clicks the 'Document' menu item in the 
         *              'Box Selection Mode' drop down menu button list when the box
         *              selection mode was 'Checkbox Control' mode.
         *      
         *          4) Calc Helper user clicks the 'Checkbox Control' menu item in the 
         *              'Box Selection Mode' drop down menu button list.
         *              
         *          5) Calc Helper entered 'lock' mode.
         * 
         *      Here is what we do to Switch From 'Checkbox Control' box selection mode:
         * 
         *          We disable the 'Checkbox Control' menu item in the 'Box Selection Mode'
         *              drop down menu button list.
         *          
         *          We set page navigation mode to pages with boxes.
         *
         *          We reflect the switch from 'Checkbox Control' box selection mode in the
         *              'Box Selection Mode' drop down button's tooltip text and image
         *              to 'No Selection' if the 'setBoxSelMode' arg is 'true'.
         *          
         *          We set the ctClient's box selection destination to 'off' if the
         *              'setBoxSelMode' arg is 'true'.
         * 
         *          We conditionally inform the Checkbox Control tool via ctClient that it
         *              will no longer receive checkbox selections. We use the 'informTool'
         *              argument to determine this.
         */
        public void ProcessChkBoxCtrlMode(bool status, bool informTool, bool setBoxSelMode)
        {
            if (ctClient == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null ctClient:\n\nProcessChkBoxCtrlMode() has detected a null ctClient.\n\nThe Checkbox Control box selection mode cannot be processed."))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            if (status == true)
            {
                // Switching To 'Checkbox Control' box selection mode:

                /*
                 * We will restore these when we are no longer sending
                 * checkbox selections to Checkbox Control.
                 * 
                 * Form logic will update the button images, for now
                 * they will remain as they are.
                 * ********************
                 */
                ctClient.refreshBtnState = (refreshBtn.Enabled == true ? btnState.on : btnState.off);
                ctClient.orphanBtnState = (orphanBoxesBtn.Enabled == true ? btnState.on : btnState.off);

                refreshBtn.Enabled = false;
                orphanBoxesBtn.Enabled = false;

                /*
                 * ********************
                 */

                chkBoxCtrlToolStripMenuItem.Enabled = true;

                if (setBoxSelMode == true)
                {
                    ProcessBoxSelMode(boxSelDest.chkBoxCtrl);
                }

                /*
                 * *********************************
                 * Just selecting checkboxes for Checkbox Control only.
                 */
                if (toolBelt != null)
                {
                    toolBelt.Close();
                }

                toolsToolStripMenuItem.Enabled = false;

                /*
                 * *********************************
                 */

                if ((pdf1 != null) && (chkboxPageToolStripMenuItem.Enabled == true))
                {
                    if (pdf1.pageNav == Pdf.pageNavMode.checkbox)
                    {
                        // Show the first checkbox page:
                        pdf1.boxedPageIndex = 0;
                        pdf1.curPage = pdf1.boxedPageList[0];
                        pdf1.ShowPage(ref docFile.hPdf, (int)pdf1.curPage, true, ref pa1);

                        ProcessPageNum(ref pdf1);
                    }
                    else
                    {
                        chkboxPageToolStripMenuItem.PerformClick();
                    }
                }

                if ((splitScreen == true) && (pdf2 != null) && (chkboxToolStripMenuItem2.Enabled == true))
                {
                    if (pdf2.pageNav == Pdf.pageNavMode.checkbox)
                    {
                        // Show the first checkbox page:
                        pdf2.boxedPageIndex = 0;
                        pdf2.curPage = pdf2.boxedPageList[0];
                        pdf2.ShowPage(ref docFile.hPdf, (int)pdf2.curPage, true, ref pa2);

                        ProcessPageNum(ref pdf2);
                    }
                    else
                    {
                        chkboxToolStripMenuItem2.PerformClick();
                    }
                }
            }
            else
            {
                /*
                 * Switching From 'Checkbox Control' box selection mode - we are
                 * no longer going to send box selections to this tool:
                 */
                chkBoxCtrlToolStripMenuItem.Enabled = false;

                /*
                 * A vscodeLock of true will disable these buttons.
                 */
                //if (ctClient.vscodeLock == false)
                //{
                    refreshBtn.Enabled = (ctClient.refreshBtnState == btnState.on ? true : false);
                    orphanBoxesBtn.Enabled = (ctClient.orphanBtnState == btnState.on ? true : false);
                //}

                if (setBoxSelMode == true)
                {
                    ProcessBoxSelMode(boxSelDest.off);
                }

                if ((pdf1 != null) && (boxedPageToolStripMenuItem.Enabled == true))
                {
                    boxedPageToolStripMenuItem.PerformClick();
                }

                if ((splitScreen == true) && (pdf2 != null) && (boxedPageToolStripMenuItem2.Enabled == true))
                {
                    boxedPageToolStripMenuItem2.PerformClick();
                }

                if (informTool == true)
                {
                    ctClient.BoxSelOff(boxSelDest.chkBoxCtrl);

                    /*
                     * Calc Helper will set the checkbox control state list
                     * from the checkbox control current list as a precaution.
                     * 
                     * A Scenario:
                     * 
                     * Checkbox Control user is clicking/lassoing away
                     * on Calc Helper adding and deleting boxes to/from checkbox
                     * control. Then they want to see a tiny checkbox's box number
                     * for some reason, so they click on the 'Box Selection Mode'
                     * menu button and select the box magnification selector
                     * which shuts down sending checkbox selections to Checkbox
                     * Control. Then the user wants to resume receiving checkbox
                     * selections from Calc Helper so they relink and Calc Helper
                     * goes and reads the checkbox sets from the editor code file
                     * and now we are out of synch, could have boxes on Calc
                     * Helper that LOOK uncontrolled but Calc Helper 'thinks'
                     * they are. Just an example.
                     */
                    SetCBCStateList("", true);
                }

                if (ctClient.vscodeLock == false)
                {
                    toolsToolStripMenuItem.Enabled = true;
                }
            }
        }

        /*
         * This handles either the switching to 'Dataflow Retriever' box selection
         * mode or switching from 'Dataflow Retriever' box selection mode.
         * 
         * We switch to this mode only as a result of a request from the Dataflow
         * Retriever tool which can be made regardless of the document being in a 
         * 'locked' or 'unlocked' state [ctClient.vscodeLock].
         * 
         * This state pertains to the ability of Calc Helper being able to send
         * box selections to the Vscode file, or to insert a code snippet into
         * the file, or to ask the Calc Tool server to take us 'somewhere' in
         * the file.
         * 
         * Called From:
         * 
         *  docBoxSelToolStripMenuItem_Click()
         *  dfRetToolStripMenuItem_Click()
         *  ctClient_CtClientEvent()
         * 
         * Arguments:
         * 
         *  status - 'true' means switching to, 'false' means switching from.
         *  
         *  informTool - 'true' means tell ctClient to send message to Dataflow Retriever
         *      that it will no longer be getting Calc Helper box selections. Used
         *      only when the 'turning off' of box selections does not come from
         *      the Dataflow Retriever tool itself, and only used when the 'status'
         *      argument is 'false'.
         *  
         *  setBoxSelMode - 'true' means we call ProcessBoxSelMode(), 'false' means we do
         *      not call it because we will be calling it for a different box selection mode
         *      so no sense in setting up for one mode then immediately change to another mode.
         *      
         * Switching To 'Dataflow Retriever' box selection mode:
         * 
         *      We enable the 'Dataflow Retriever' menu item in the 'Box Selection Mode'
         *          drop down menu button list. Users will now be able to turn this
         *          mode off from Calc Helper.
         *          
         *      We set page navigation mode to pages with boxes to make it easier
         *          for the user to select them [if not already in that mode].
         *          We do this for both display panels when a splitscreen.
         *          
         *      We reflect the switch to 'Dataflow Retriever' box selection mode in the
         *          'Box Selection Mode' drop down button's tooltip text and image if
         *          the 'setBoxSelMode' arg is 'true'.
         *          
         *      We set the ctClient's box selection destination to 'Dataflow Retriever'
         *          if the 'setBoxSelMode' arg is 'true'.
         *          
         * Switching From 'Dataflow Retriever' box selection mode (i.e. no longer doing it):
         * 
         *      These are the scenarios of how we got to turning this setting off.
         *  
         *          1) We receive from ctClient an accepted request from another tool
         *              for box selections. Accepted means both the other tool
         *              and Calc Helper need to be linked to the same document file.
         *              ctClient accepts or rejects.
         *      
         *          2) We receive from ctClient notification that the Dataflow Retriever
         *              tool no longer wants box selections from Calc Helper.
         *              
         *          3) Calc Helper user clicks the 'Document' menu item in the 
         *              'Box Selection Mode' drop down menu button list when the box
         *              selection mode was 'Dataflow Retriever' mode.
         *      
         *          4) Calc Helper user clicks the 'Dataflow Retriever' menu item in the 
         *              'Box Selection Mode' drop down menu button list.
         * 
         *      Here is what we do to Switch From 'Dataflow Retriever' box selection mode:
         * 
         *          We disable the 'Dataflow Retriever' menu item in the 'Box Selection Mode'
         *              drop down menu button list.
         *          
         *          We reflect the switch from 'Dataflow Retriever' box selection mode in the
         *              'Box Selection Mode' drop down button's tooltip text and image
         *              to 'No Selection' if the 'setBoxSelMode' arg is 'true'.
         *          
         *          We set the ctClient's box selection destination to 'off' if the
         *              'setBoxSelMode' arg is 'true'.
         * 
         *          We conditionally inform the Dataflow Retriever tool via ctClient that it
         *              will no longer receive box selections. We use the 'informTool'
         *              argument to determine this.
         */
        public void ProcessDFRetMode(bool status, bool informTool, bool setBoxSelMode)
        {
            if (ctClient == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null ctClient:\n\nProcessDFRetMode() has detected a null ctClient.\n\nThe Dataflow Retriever box selection mode cannot be processed."))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            if (status == true)
            {
                // Switching To 'Dataflow Retriever' box selection mode:

                dfRetToolStripMenuItem.Enabled = true;

                if (setBoxSelMode == true)
                {
                    ProcessBoxSelMode(boxSelDest.dfRetriever);
                }

                /*
                 * *********************************
                 * Just selecting boxes for the Dataflow Retriever only.
                 */
                if (toolBelt != null)
                {
                    toolBelt.Close();
                }

                toolsToolStripMenuItem.Enabled = false;

                /*
                 * *********************************
                 */

                if ((pdf1 != null) && (boxedPageToolStripMenuItem.Enabled == true))
                {
                    boxedPageToolStripMenuItem.PerformClick();
                }

                if ((splitScreen == true) && (pdf2 != null) && (boxedPageToolStripMenuItem2.Enabled == true))
                {
                    boxedPageToolStripMenuItem2.PerformClick();
                }
            }
            else
            {
                /*
                 * Switching From 'Dataflow Retriever' box selection mode - we are
                 * no longer going to send box selections to this tool:
                 */
                dfRetToolStripMenuItem.Enabled = false;

                if (setBoxSelMode == true)
                {
                    ProcessBoxSelMode(boxSelDest.off);
                }

                if (informTool == true)
                {
                    ctClient.BoxSelOff(boxSelDest.dfRetriever);
                }

                if (ctClient.vscodeLock == false)
                {
                    toolsToolStripMenuItem.Enabled = true;
                }
            }
        }

        /*
         * This checks if we need to refresh a panel [pdf1 or pdf2] after
         * a Refresh event has been processed. A Refresh event is where we read
         * the document's Javascript file. We also do this after processing Checkbox
         * Sets received from Checkbox Control.
         * 
         * This is so we can properly display the latest calc controlled boxes like
         * Calcs, Dataflow, Checkbox Control and Bluefield Control.
         * 
         * Called From:
         * 
         * refreshBtn_Click()
         * ctClient_CtClientEvent()
         * toolBelt_ToolBeltEvent()
         * ProcessBoxSelMode()
         * ProcessLeftMouseClick()
         * ProcessLeftMouseUp()
         */
        public void RefreshPanels()
        {
            // Caller has ensured pdf1 and pa1 are not null.

            /*
             * Check if we should show this page to reflect any controlled box changes.
             */
            if (pdf1.refreshPage == true)
            {
                pdf1.centerPic = false;
                pdf1.ShowPage(ref docFile.hPdf, (int)pdf1.curPage, false, ref pa1);
                pdf1.centerPic = true;
                pdf1.refreshPage = false;
            }

            /*
             * I think back around 2 years I had Calc Helper allowing a splitscreen
             * with empty panels because I had file open/close capability for each.
             * Just being cautious here by all of this checking even though we can
             * no longer open or close a file via a menu or button selection.
             */
            if (splitScreen == true)
            {
                if ((pdf2 != null) && (pa2 != null))
                {
                    if (pdf2.refreshPage == true)
                    {
                        pdf2.centerPic = false;
                        pdf2.ShowPage(ref docFile.hPdf, (int)pdf2.curPage, false, ref pa2);
                        pdf2.centerPic = true;
                        pdf2.refreshPage = false;
                    }
                }
            }
        }

        /*
         * This processes right mouse clicks as opposed to left mouse
         * clicks, you know how those mice are. Clicks, no lassoing.
         * 
         * Right mouse clicks are for the following actions:
         * 
         * Right clicking on a controlled box will take the user to the
         * box's controlling code in Vscode. Controlling code consists
         * of Dataflow, Calcs, Checkbox Control and Bluefield Control.
         * 
         * Right clicking a non-controlled box will signal that this
         * box is being calculated and the box will be passed on to
         * Vscode [unformatted, 'box1' for example] for insertion
         * in the editor at the cursor location, which may be inside 
         * a code snippet. The box will be colored to indicate it is
         * being calculated.
         * 
         * Called From:
         * 
         *      pictureBox1_MouseClick()
         *      pictureBox2_MouseClick()
         */
        public void ProcessRightMouseClick(ref Pdf pdf, ref PageArray pa, Point location)
        {
            /*
             * Caller has ensured that pdf, pa, and ctClient are not null, and
             * it is a right mouse button event. Squeak.
            */

            List<Field> fa = null;
            RightCtrlBoxSel ctrlBoxSel;
            RightCalcBoxSel calcBoxSel;
            bool goToCtrlCode = false;
            bool doCalcBox = false;
            int paIndex = (int)(pdf.curPage - 1);

            pa.GetFieldArrayRef(paIndex, ref fa);

            if (fa != null)
            {
                for (int i = 0; i < fa.Count; i++)
                {
                    if (fa[i].sbox.Contains(location))
                    {
                        /*
                         * We have determined that a box has been clicked on the current
                         * page. Now we need to see if the box is a controlled box or not.
                         */
                        if (fa[i].ctrlType < 0)
                        {
                            /*
                             * Not controlled. Need to be in document box selection mode
                             * to proceed with making this a calculating box.
                             */
                            if (ctClient.boxSelClient == boxSelDest.document)
                            {
                                calcBoxSel = new RightCalcBoxSel(fa[i].ref_name, paIndex, i);

                                /*
                                 * Set args in this order: (calcBoxSel, pdf1, pa1, pdf2, pa2)
                                 * for either one below.
                                 */
                                if (pdf.id == "pdf1")
                                {
                                    doCalcBox = boxSel.ProcessRightCalcBoxSel(calcBoxSel, ref pdf, ref pa, ref pdf2, ref pa2);
                                }
                                else
                                {
                                    /*
                                     * In this case the caller verified that pdf2 and pa2 are not null,
                                     * we will check pdf1 and pa1 here just to be safe, should be ok.
                                     */
                                    if (Pa1OrPdf1AreNull("ProcessRightMouseClick", "Cannot process right calc box selection.") == true)
                                    {
                                        return;
                                    }

                                    doCalcBox = boxSel.ProcessRightCalcBoxSel(calcBoxSel, ref pdf1, ref pa1, ref pdf, ref pa);
                                }

                                if (doCalcBox)
                                {
                                    RefreshPanels();

                                    /*
                                     * Send on to Vscode? i.e. Did we select the box
                                     * as opposed to unselecting it?
                                     */
                                    if (boxSel.toVscode == true)
                                    {
                                        boxSel.toVscode = false;
                                        ctClient.SendBoxSelections(calcBoxSel.box);
                                    }
                                }
                            }
                            else
                            {
                                string msg = "Calculating Box Selection Error:\n\nCalc Helper must be in Document Box Selection Mode before you can select a box to calculate.";

                                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                                {
                                    mainMsg.ShowDialog(this);
                                }
                            }
                        }
                        else
                        {
                            /*
                             * First process this controlled box selection:
                             */
                            ctrlBoxSel = new RightCtrlBoxSel(fa[i].ref_name, paIndex, i, fa[i].ctrlType);

                            if (pdf.id == "pdf1")
                            {
                                goToCtrlCode = boxSel.ProcessRightCtrlBoxSel(ctrlBoxSel, ref pdf, ref pa, ref pdf2, ref pa2, ref docFile);
                            }
                            else
                            {
                                goToCtrlCode = boxSel.ProcessRightCtrlBoxSel(ctrlBoxSel, ref pdf, ref pa, ref pdf1, ref pa1, ref docFile);
                            }

                            /*
                             * Take user to Vscode control code location(s) for this
                             * box if applicable:
                            */
                            if (goToCtrlCode == true)
                            {
                                GotoControlCode(ctrlBoxSel);
                            }
                        }

                        break;
                    }
                }
            }
        }

        /*
         * This determines the type of controlled box we are
         * going to find the control code for in Vscode. Refer
         * to the switch statement below for the types.
         * 
         * We prepare a string using the box as well as regex
         * code to be used by the find widget in Vscode.
         * We use regex code since it will pass over other code
         * in the Vscode file that would otherwise give us false
         * 'finds' which would be bothersome to the user as they
         * would need to select the next find when there are
         * multiple hits.
         * 
         * NOTE: ctrlBoxSel.ctrlType values also are indices
         * into the Pdf.cs 'ctrlColors' BRUSH array which is
         * used in filling the fields with the control color.
         * 
         * Called From:
         * 
         *      ProcessRightMouseClick()
         *      
         * Arguments:
         * 
         *      ctrlBoxSel - Holds our box number and the ctrlType
         *          which identifies what type of control code
         *          we will be looking for in Vscode.
         */
        public void GotoControlCode(RightCtrlBoxSel ctrlBoxSel)
        {
            string ctrlStr = "";

            switch (ctrlBoxSel.ctrlType)
            {
                case (0):

                    /*
                     * Checkbox Control:
                     * 
                     * The Calc Tool Server first scrolls up to line 1 in the document
                     * so we can do a simple search here for just the box. We close
                     * the find widget so that we do not find the other locations where
                     * this box will definitely be found. Also finding just the box means
                     * the find operation will work regardless of the find widget using
                     * regular expression mode.
                     */

                    // THIS FIRST TRY HOWEVER IS A GREAT WAY TO SELECT THE WHOLE FUNCTION OR
                    // JUST THE RETURN LINE CONTAINING THE BOX ARRAY - CAN USE THIS IN CHECKBOX
                    // CONTROL AFTER THE USER CLICKS THE COPY TO CLIPBOARD BUTTON, WE CAN
                    // AUTOMATICALLY SELECT THIS FOR THEM THEN THEY CAN SAFELY CTRL-V OVER IT.
                    // WILL NEED TO COMPLETE THE WHOLE SELECTION REGEX IN THIS THOUGH, IT ONLY
                    // SELECTS UP TO AND INCLUDING OUR SEARCH BOX.

                    //ctrlStr = string.Format("^\\s*function\\s+getCheckBoxList\\s*\\(\\s*\\)\\s*{{\\n\\s*return\\s*\\[.+[\"']{0}[\"']", ctrlBoxSel.box);

                    ctrlStr = ctrlBoxSel.box;

                    break;

                case (1):

                    // Bluefield Control - shares same format as Calcs:
                    ctrlStr = string.Format("^\\s*{0}\\s*=(?!=).+", ctrlBoxSel.box);

                    break;

                case (2):

                    /*
                     * Dataflow: Yeah, I know..ugly, but Dataflow has two types that I look for.
                     * Here are two simple examples, not relying on 'tmp' always being used,
                     * this is just an example:
                     * 
                     * SFForms.StorageManager.setValue(formInstanceId, "box1_1", tmp);
                     * SFForms.StorageManager.setValue(formInstanceId, (addr2.trim().isEmpty() ? 'box1_3' : 'box1_4'), tmp);
                     */
                    if (docFile.newBoxFormat == true)
                    {
                        ctrlStr = string.Format("\\s*SFForms\\.StorageManager\\.setValue\\(\\s*formInstanceId\\s*,(\\s*[\"']{0}[\"'](?!\\s*,\\s*({0}|\"\")))|(((.+\\?\\s*[\"']{0}[\"']\\s*:\\s*[\"']box\\d+[\"'])|(.+\\?\\s*[\"']box\\d+[\"']\\s*:\\s*[\"']{0}[\"']))(?!\\s*\\)?\\s*,\\s*box\\d+))", ctrlBoxSel.box);
                    }
                    else
                    {
                        ctrlStr = string.Format("\\s*SFForms\\.StorageManager\\.setValue\\(\\s*formInstanceId\\s*,(\\s*[\"']{0}[\"'](?!\\s*,\\s*({0}|\"\")))|(((.+\\?\\s*[\"']{0}[\"']\\s*:\\s*[\"']box\\d+_\\d+[\"'])|(.+\\?\\s*[\"']box\\d+_\\d+[\"']\\s*:\\s*[\"']{0}[\"']))(?!\\s*\\)?\\s*,\\s*box\\d+_\\d+))", ctrlBoxSel.box);
                    }

                    break;

                case (3):

                    // Calcs - Shares same format as Bluefield control:
                    ctrlStr = string.Format("^\\s*{0}\\s*=(?!=).+", ctrlBoxSel.box);

                    break;
            }

            if (ctrlStr.Length > 0)
            {
                /*
                 * A little clunky but ctrlType 0 [Checkbox
                 * Control] can safely be used when the find widget
                 * in Vscode is set for Regex finds or not as we
                 * only search for the box itself. So we close
                 * the find widget.
                 * 
                 * The other types are searched for with the find
                 * widget left open as there can be more than one
                 * code location to go to. These ctrlType's use
                 * regular expressions in the search string.
                 */
                if (ctrlBoxSel.ctrlType > 0)
                {
                    ctClient.GotoCode(false, false, ctrlStr);
                }
                else
                {
                    ctClient.GotoCode(true, true, ctrlStr);
                }
            }
        }

        /*
         * ToolBelt event handler. The ToolBelt sends messages (events)
         * to the form concerning box selection reset/aborts:
         * 
         * tbEvent.clearAllSel:
         *      Unselect ALL selected boxes.
         *      
         * tbEvent.clearOpWaitSel:
         *      Unselect the boxes that the Tool Belt's Box Selector
         *      is holding onto while waiting for the user to choose
         *      an operator or separator to apply to them before
         *      sending on to Vscode. These are left mouse selection.
         *
         * tbEvent.clearBoxSel:
         *      Unselect certain left mouse selected boxes due to
         *      some kind of error. Should not happen, but...
         */
        void toolBelt_ToolBeltEvent(object sender, ToolBeltArgs e)
        {
            if (e != null)
            {
                // These pa1 and pdf1 cannot be null, else we abort this event.
                if (Pa1OrPdf1AreNull("toolBelt_ToolBeltEvent", "Can not handle the Tool Belt event.") == true)
                {
                    return;
                }

                switch (e.eventType)
                {
                    case (tbEvent.clearAllSel):

                        //MessageBox.Show("  ToolBelt wants all selections cleared!", "Clear All Selections:", MessageBoxButtons.OK, MessageBoxIcon.Information);

                        boxSel.ResetSelections(boxSelTypes.allSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                        break;

                    case (tbEvent.clearOpWaitSel):

                        boxSel.ResetSelections(boxSelTypes.leftToolBeltOpWaits, ref pa1, ref pdf1, ref pa2, ref pdf2);
                        break;

                    case (tbEvent.clearBoxSel):

                        boxSel.unselLeftBoxes.AddRange(e.boxSel);
                        boxSel.ResetSelections(boxSelTypes.leftSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                        break;
                }

                RefreshPanels();
            }
        }

        /*
         * This checks to see if either pdf1 or pa1 are null,
         * if so we return false. Using this function as this 
         * check is performed several times and results in a
         * lot of code being repeated over and over.
         * 
         * Mainly used for selection resetting. These really
         * should not be null as these are the main components
         * of Calc Helper, pdf2 and pa2 are used when we have
         * a split screen, but we should always have non-null
         * instances of pdf1 and pa1 - I want to know if they
         * ever are null rather than having whatever was going
         * to be done silently not done.
         * 
         * Called From:
         *      ProcessDocLock()
         *      ProcessBoxSelMode()
         *      toolBelt_ToolBeltEvent()
         *      refreshBtn_Click()
         *      ToolBeltClosed()
         *      ProcessLeftMouseUp()
         *      ProcessLeftMouseClick()
         *      ProcessRightMouseClick()
         */
        public bool Pa1OrPdf1AreNull(string fcnName, string arg1)
        {
            bool result = false;
            string msg = "";

            if (pa1 == null)
            {
                msg = string.Format("null pa1:\n\n{0}() has detected a null pa1.\n\n{1}", fcnName, arg1);

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                result = true;
            }
            else if (pdf1 == null)
            {
                msg = string.Format("null pdf1:\n\n{0}() has detected a null pdf1.\n\n{1}", fcnName, arg1);

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                result = true;
            }

            return result;
        }

        /*
         * User can select to enable Document Box Selection Mode at startup,
         * or to disable it in which case it will be 'No Selection'. 
         * 
         * Called From:
         * 
         *      docBoxSelModeAtStartupToolStripMenuItem_Click()
         */
        public void ProcessDocBoxSelStartupMode(bool enable)
        {
            if (enable)
            {
                Properties.Settings.Default.DocBoxSelMode = true;
                docBoxSelModeAtStartupToolStripMenuItem.Text = "Disable 'Document Box Selection Mode' At Startup";
            }
            else
            {
                Properties.Settings.Default.DocBoxSelMode = false;
                docBoxSelModeAtStartupToolStripMenuItem.Text = "Enable 'Document Box Selection Mode' At Startup";
            }

            Properties.Settings.Default.Save();
        }

        /*
         * User has clicked with the left mouse button AND 'Box Selection
         * Mode' is boxNumMag. This mode 'locks out' all other mouse
         * click code and operates regardless of a document lock. It will
         * be automatically switched off if another tool requests box
         * selections.
         * 
         * This click event is handled differently than the other clicks
         * [and lassos] as they are for selecting a box to send to Vscode
         * or to another tool, or in the case of a right click on a
         * controlled box to goto the controlling code in Vscode.
         * 
         * This method will always show the box number in a tooltip popup
         * and will draw the selection rectangle if it is the first time
         * the user clicked on the box [what I mean is if someone just
         * clicks the same box over and over]. If the user subsequently
         * clicks on a different box then the previous clicked box will
         * have it's selection rectangle removed. 
         * 
         * Called By:
         * 
         *  pictureBox1_MouseClick()
         *  pictureBox2_MouseClick()
         */
        public void ProcessLeftMouseBoxNumMagClick(ref Pdf pdf, ref PageArray pa, Point location)
        {
            /*
             * Caller has ensured that pdf, pa, and ctClient are not null,
             * and it is a left mouse button event.
            */
            List<Field> fa = null;
            List<LeftBoxSel> boxSelList = new List<LeftBoxSel>();
            LeftBoxSel lbs;
            bool doRefresh = false;
            int paIndex = (int)(pdf.curPage - 1);
            int boxMagIndex = -1;
            string magBox = "";

            pa.GetFieldArrayRef(paIndex, ref fa);

            if (fa != null)
            {
                for (int i = 0; i < fa.Count; i++)
                {
                    if (fa[i].sbox.Contains(location))
                    {
                        magBox = fa[i].ref_name;
                        boxMagIndex = i;
                        break;
                    }
                }

                if (magBox.Length > 0)
                {
                    /*
                     * ProcessBoxSelMode() resets ALL selections when we change
                     * box selection mode, including switching to this mode, so
                     * we will have just 1 element in this list.
                     */
                    if ((boxSel.unselLeftBoxes.Count > 0) && (!boxSel.unselLeftBoxes.Contains(magBox)))
                    {
                        // Reset the previously selected magnification box.
                        if (Pa1OrPdf1AreNull("ProcessLeftMouseClick", "Can not handle box unselection.") == false)
                        {
                            boxSel.ResetSelections(boxSelTypes.leftSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                            doRefresh = true;
                        }
                    }

                    /*
                     * Now select this box if not already selected:
                     */
                    if (!boxSel.leftBoxSelDict.ContainsKey(magBox))
                    {
                        lbs = new LeftBoxSel(magBox, paIndex, boxMagIndex);
                        boxSelList.Add(lbs);

                        if (boxSel.ProcessLeftBoxSel(ref boxSelList, ref pdf1, ref pa1, ref pdf2, ref pa2) == true)
                        {
                            boxSel.unselLeftBoxes.Add(magBox);
                            doRefresh = true;
                        }
                    }

                    if (doRefresh == true)
                    {
                        RefreshPanels();
                    }

                    // Now show the box number in the magnification tool tip window:
                    if (pdf.id == "pdf1")
                    {
                        boxNumMagToolTip.SetToolTip(this.pictureBox1, magBox);
                    }
                    else
                    {
                        boxNumMagToolTip.SetToolTip(this.pictureBox2, magBox);
                    }
                }
            }
        }

        private void boxNumMagToolTip_Popup(object sender, PopupEventArgs e)
        {
            if ((e.AssociatedControl == pictureBox1) || (e.AssociatedControl == pictureBox2))
            {
                using (Font f = new Font("Microsoft Sans Serif", 18.0f, FontStyle.Bold))
                {
                    e.ToolTipSize = TextRenderer.MeasureText(
                        boxNumMagToolTip.GetToolTip(e.AssociatedControl), f);
                }
            }
        }

        private void boxNumMagToolTip_Draw(System.Object sender,
                    System.Windows.Forms.DrawToolTipEventArgs e)
        {
            if ((e.AssociatedControl == pictureBox1) || (e.AssociatedControl == pictureBox2))
            {
                // Draw custom background.
                e.Graphics.FillRectangle(SystemBrushes.Info, e.Bounds);

                // Draw the custom border to appear 3-dimensional.
                e.Graphics.DrawLines(SystemPens.ControlLightLight, new Point[] {
                    new Point (0, e.Bounds.Height - 1), 
                    new Point (0, 0), 
                    new Point (e.Bounds.Width - 1, 0)
                });
                e.Graphics.DrawLines(SystemPens.ControlDarkDark, new Point[] {
                    new Point (0, e.Bounds.Height - 1), 
                    new Point (e.Bounds.Width - 1, e.Bounds.Height - 1), 
                    new Point (e.Bounds.Width - 1, 0)
                });

                // Draw the custom text.
                // The using block will dispose the StringFormat automatically.
                using (StringFormat sf = new StringFormat())
                {
                    sf.Alignment = StringAlignment.Center;
                    sf.LineAlignment = StringAlignment.Center;
                    sf.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.None;
                    sf.FormatFlags = StringFormatFlags.NoWrap;
                    using (Font f = new Font("Microsoft Sans Serif", 18.0f, FontStyle.Bold))
                    {
                        e.Graphics.DrawString(e.ToolTipText, f,
                            SystemBrushes.ActiveCaptionText, e.Bounds, sf);
                    }
                }
            }
        }

        /*
         * This takes the CalcHelper copy of the Checkbox
         * Control box sets [as received from the connected
         * Visual Studio Code file] and stores them in the
         * Calc Helper global copy cbcBoxSetsList. What 
         * happens to the boxes in this set regarding
         * additions, deletions will be mirrored in
         * Checkbox Control.
         * 
         * Duplicate and orphan boxes are removed before 
         * being sent over to Checkbox Control.
         * 
         * Called By:
         * 
         *      Form1.cs-ctClient_CtClientEvent()
         *     
         * Globals Affected:
         * 
         *      boxSel.cbcBoxSetsList
         */
        public void ProcessCBCSelReq()
        {
            Match match;
            string tmpStr = "";
            string tmpBoxStr;
            List<string> tmpSetList = new List<string>();
            List<string> tmpBoxList = new List<string>();
            List<string> tmpBoxFiltList = new List<string>();

            boxSel.cbcBoxSetsList.Clear();

            tmpStr = Regex.Replace(docFile.vscodeChkBoxSets, "[\n\r\t ]", "");

            if (docFile.newBoxFormat == true)
            {
                match = Regex.Match(tmpStr, @"return\[(\[""box\d+""(,""box\d+"")*\](,\[""box\d+""(,""box\d+"")*\])*)\];");
            }
            else
            {
                match = Regex.Match(tmpStr, @"return\[(\[""box\d+_\d+""(,""box\d+_\d+"")*\](,\[""box\d+_\d+""(,""box\d+_\d+"")*\])*)\];");
            }

            if (match.Success)
            {
                /*
                 * Using '!' as a placeholder so we can split the string into individual box
                 * sets for displaying in the setList.
                 */
                tmpSetList = match.Groups[1].Value.Replace("\"", "").Replace(",[", "![").Split('!').ToList();

                tmpStr = "";

                for (int i = 0; i < tmpSetList.Count; i++)
                {
                    // Remove any orphan checkboxes.
                    tmpBoxFiltList.Clear();

                    // Remove any duplicate boxes [should not be any, but...] from each set.
                    tmpBoxList = tmpSetList[i].Replace("[", "").Replace("]", "").Split(',').Distinct().ToList();

                    foreach (string box in tmpBoxList)
                    {
                        if (docFile.docBoxList.Contains(box))
                        {
                            tmpBoxFiltList.Add(box);
                        }
                    }

                    if (tmpBoxFiltList.Count > 0)
                    {
                        tmpBoxStr = string.Join(",", tmpBoxFiltList.ToArray());
                        tmpStr += "[" + tmpBoxStr + "]!";

                        boxSel.cbcBoxSetsList.Add(tmpBoxStr);
                    }
                }
            }
            else
            {
                tmpStr = "";
            }

            /*
             * We call ctClient.SendBoxSelections() even if we have no box sets.
             * This is because we MUST maintain box set congruency with Checkbox
             * Control in all cases. So if, for some weird reason, Checkbox
             * Control has box sets and Calc Helper does not this will result
             * in Checkbox Control box sets being cleared to stay at 1:1 with
             * Calc Helper. Calc Helper gets its box sets from the same file
             * Checkbox Control was launched from BUT it is possible [unlikely]
             * that Checkbox Controls box sets which are copied from the clipboard
             * got copied into the clipboard from a different file. Rare case, but
             * in pure logic we have to cover the 'what ifs...'.
             */

            /*
             * Currently can not edit the Calc Tool Server
             * [guerilla coding!] so making use of the current
             * message commands. Using '!' so Checkbox Control
             * can split the sting into sets of boxes.
             */
            ctClient.SendBoxSelections(string.Format("{0}|{1}", cbcBoxSelTypes.boxSets, tmpStr.TrimEnd('!')));

            boxSel.cbcCurIndex = -1;
            boxSel.cbcCurPage = -1;
        }

        /*
         * The Refresh button click routine.
         * 
         * Called By:
         * 
         * refreshBtn_Click()
         * ctClient_CtClientEvent()
         */
        public void ProcessRefreshBtnClick()
        {
            refreshBtn.Image = CalcHelper.Properties.Resources.RefreshOff;
            refreshBtn.Enabled = false;
            ctClient.refreshBtnState = btnState.off;

            if (docFile == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null docFile:\n\nrefreshBtn_Click() has detected a null docFile.\n\nRefresh will not be possible."))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // These pa1 and pdf1 cannot be null, else we abort this function.
            if (Pa1OrPdf1AreNull("refreshBtn_Click", "Refresh will not be possible.") == true)
            {
                return;
            }

            /*
             * 'true' indicates this is a Refresh, as opposed to 'we just opened
             * the file'. We pass the page array args to set the fields on the
             * pages where we will indicate any controlled boxes, like dataflow
             * or calcs for example.
             * 
             * We then reset all selections on the document, and refresh the panels
             * to reflect the now current controlled boxes and unselections.
             * 
             * Lastly, we set the Tool Belt's Code Navigator lists to reflect any
             * changes to it's lists.
             */
            if (docFile.ProcessJSFile(true, ref pa1, ref pdf1, ref pa2, ref pdf2) == true)
            {
                boxSel.ResetSelections(boxSelTypes.allSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                RefreshPanels();
                docFile.SetCodeNavLists();

                ProcessOrphanBoxesButton();
            }
        }
    }
}